'use strict';
var Site = require('dw/system/Site'),
    currentSite = Site.getCurrent();

var scalapayUtils = require('*/cartridge/scripts/helpers/scalapayUtils');
var settingsMapping = require('*/cartridge/scripts/settings/settingsMapping');

/**
 * @description generate order address simple object
 * @param {dw.order.OrderAddress} orderAddress SFCC basket/order address
 * @returns {Object} order address simple object
 */
function getAddressData(orderAddress) {
    var addressObj = {};

    if (orderAddress) {
        addressObj.name = orderAddress.firstName + ' ' + orderAddress.lastName;
        addressObj.line1 = orderAddress.address1 || '';

        addressObj.postcode = orderAddress.postalCode || '';
        addressObj.countryCode = orderAddress.countryCode ? orderAddress.countryCode.value.toUpperCase() : '';
        addressObj.phoneNumber = orderAddress.phone || '';
        addressObj.suburb = orderAddress.city || '';
    }
    return addressObj;
}

/**
 * @description get basket price adjustments, generate adjustments simple data array
 * @param {dw.order.LineItemCtnr} basket SFCC basket/order
 * @returns {Array} basket price adjustments simple objects array
 */
function getPriceAdjustmentData(basket) {
    var priceAdjustmentsArr = [];

    if (basket) {
        var priceAdjustments = basket.getPriceAdjustments();

        for (var i = 0; i < priceAdjustments.size(); i += 1) {
            var priceAdjustmentObj = priceAdjustments[i],
                couponLineItem = priceAdjustmentObj.getCouponLineItem();

            if (couponLineItem) {
                var promoObj = priceAdjustmentObj.getPromotion();

                if (promoObj) {
                    var promoName = promoObj.name;

                    var priceAdjObj = {
                        displayName: (promoName || 'Not Set'),
                        amount: {
                            amount: (-priceAdjustmentObj.priceValue) + '',
                            currency: session.currency.currencyCode
                        }
                    };
                    priceAdjustmentsArr.push(priceAdjObj);
                }
            }
        }
    }

    return priceAdjustmentsArr;
}

/**
 * @description get product categories path
 * @param {dw.catalog.Product} pliProduct SFCC product object
 * @returns {Array} product categories path arrat
 */
function getProductCategoryRootPath(pliProduct) {
    var categoriesRootPathArr = [],
        productCategory;

    if (pliProduct) {
        productCategory = pliProduct.getPrimaryCategory();

        if (!productCategory && pliProduct.isVariant()) {
            var productVariationModel = pliProduct.getVariationModel();
            if (productVariationModel) {
                productCategory = productVariationModel.getMaster().getPrimaryCategory();
            }
        }

        if (productCategory) {
            categoriesRootPathArr.push(productCategory.ID);
            if (productCategory.parent) {
                while (productCategory.parent.ID !== 'root') {
                    productCategory = productCategory.parent;
                    categoriesRootPathArr.push(productCategory.ID);
                }
            }
        }
    }
    return categoriesRootPathArr;
}

/**
 * @description get payment option external ID value
 * @param {string} paymentOptionID payment option internal ID
 * @returns {string} Payment option external ID
 */
function getPaymentProductType(paymentOptionID) {
    var paymentProductID;

    if (paymentOptionID && paymentOptionID in settingsMapping.general.paymentProductTypes) {
        paymentProductID = settingsMapping.general.paymentProductTypes[paymentOptionID];
    }
    return paymentProductID;
}

/**
 * @description Get payment option payment frequency
 * @param {string} paymentOptionID payment option internal ID
 * @returns {string} payment option payment frequency value
 */
function getPaymentFrequencySettings(paymentOptionID) {
    var frequencySettings = {
        number: 1,
        frequencyType: 'monthly'
    };

    if (paymentOptionID && paymentOptionID in settingsMapping.general.frequencySettings) {
        frequencySettings.number = settingsMapping.general.frequencySettings[paymentOptionID];
        frequencySettings.frequencyType = settingsMapping.general.frequencyType[paymentOptionID];
    }

    return frequencySettings;
}

/**
 * @description get busket items, generate simple data object
 * @param {dw.order.LineItemCtnr} basket SFCC Basket/Order object
 * @returns {Object} Basket items simple object
 */
function getItemsData(basket) {
    var itemsData = [];

    if (basket && basket.productLineItems && basket.productLineItems.size() > 0) {
        var basketItems = basket.productLineItems;

        for (var i = 0; i < basketItems.size(); i += 1) {
            var PLI = basketItems[i],
                pliProduct = PLI.product,
                productCategories = getProductCategoryRootPath(pliProduct),
                itemPrice = PLI.adjustedPrice.divide(PLI.quantityValue),
                productCategory = (productCategories.length > 0 ? productCategories[0] : '');

            var itemObj = {
                name: (PLI.productName || '').toString(),
                sku: PLI.productID,
                quantity: PLI.quantityValue,
                price: {
                    amount: itemPrice.value.toString(),
                    currency: session.currency.currencyCode
                },
                category: productCategory,
                subcategory: productCategories,
                brand: pliProduct.getBrand() || '',
                gtin: pliProduct.getEAN() || ''
            };
            itemsData.push(itemObj);
        }
    }
    return itemsData;
}

/**
 * @description get customer profile data, generate simple profile data object
 * @param {dw.order.LineItemCtnr} basket SFCC Basket/Order object
 * @returns {Object} customer profile simple data object
 */
function getCustomerData(basket) {
    var customerData;
    if (basket) {
        var phoneNumber = basket.billingAddress.phone,
            firstName = basket.billingAddress.firstName,
            lastName = basket.billingAddress.lastName;

        if (customer.profile) {
            phoneNumber = (customer.profile.phoneHome || phoneNumber);
            firstName = (customer.profile.firstName || firstName);
            lastName = (customer.profile.lastName || lastName);
        }

        customerData = {
            phoneNumber: phoneNumber,
            givenNames: firstName,
            surname: lastName,
            email: basket.customerEmail
        };
    }
    return customerData;
}

var ScalaPayHelper = {
    isEnabled: function (pageType, widgetType) {
        var isEnabled = currentSite.getCustomPreferenceValue(settingsMapping.general.availability.global);

        if (isEnabled) {
            if (widgetType) {
                isEnabled = currentSite.getCustomPreferenceValue(settingsMapping.general.availability[widgetType]);

                if (pageType) {
                    isEnabled = currentSite.getCustomPreferenceValue(settingsMapping.general.availability[pageType][widgetType]);
                }
            }
        }
        return isEnabled;
    },
    getOptionConfiguration: function (pageType, optionID) {
        var minPaymentValKey = settingsMapping.general[optionID].minPaymentVal,
            maxPaymentValKey = settingsMapping.general[optionID].maxPaymentVal,
            numberOfInstKey = settingsMapping.general[optionID].installmentsNumber;

        var configuration = {
            minimumAmount: {
                amount: currentSite.getCustomPreferenceValue(minPaymentValKey)
            },
            maximumAmount: {
                amount: currentSite.getCustomPreferenceValue(maxPaymentValKey)
            },
            promotionUrl: '',
            numberOfPayments: currentSite.getCustomPreferenceValue(numberOfInstKey).value,
            locales: [],
            isEnabled: ScalaPayHelper.isEnabled(pageType, optionID),
            frequency: settingsMapping.general.frequencySettings[optionID]
        };
        return configuration;
    },
    getWidgetText: function (amount, numberOfPayments, frequency, optionID) {
        var Resource = require('dw/web/Resource'),
            Money = require('dw/value/Money');

        var widgetText = {},
            currencySymbol = session.currency.symbol,
            moneyVal = new Money(amount, session.currency.currencyCode),
            dividedMoneyVal = moneyVal.divide(+numberOfPayments).value;

        widgetText.label = Resource.msgf('scalapay.widget.text.payment.' + optionID, 'scalapay', null, numberOfPayments, dividedMoneyVal, currencySymbol);
        widgetText.description = Resource.msgf('scalapay.payment.method.description.' + optionID, 'scalapay', null, frequency);

        return widgetText;
    },
    getScalaPayBillingSettings: function () {
        var scalaPayUtils = require('*/cartridge/scripts/helpers/scalapayUtils'),
            basketTotalVal = scalaPayUtils.getNonGiftCertificateAmount().value,
            widgetsSettings = {
                isEligible: false
            };

        settingsMapping.general.options.forEach(function (optionID) {
            widgetsSettings[optionID] = ScalaPayHelper.getScalaPayOptionSettings(basketTotalVal, null, optionID);
            if (widgetsSettings[optionID].isEligible) {
                widgetsSettings.isEligible = true;
            }
        });
        return widgetsSettings;
    },
    getScalaPayOptionSettings: function (amount, pageType, optionID) {
        var scalapayOptionSettings = {
            isEligible: false
        };

        if (amount) {
            var scalapayEnabled = ScalaPayHelper.isEnabled(pageType, optionID);

            if (scalapayEnabled && ScalaPayHelper.isCurrencyApplicable()) {
                var configuration = ScalaPayHelper.getOptionConfiguration(pageType, optionID);
                var minAmount = +configuration.minimumAmount.amount;
                var maxAmount = +configuration.maximumAmount.amount;

                if (amount >= minAmount && amount <= maxAmount) {
                    scalapayOptionSettings.isEligible = true;
                }
                scalapayOptionSettings.settings = configuration;
                scalapayOptionSettings.widgetText = ScalaPayHelper.getWidgetText(amount, configuration.numberOfPayments, configuration.frequency, optionID);
                scalapayOptionSettings.min = minAmount;
                scalapayOptionSettings.max = maxAmount;
                scalapayOptionSettings.numberOfPayments = configuration.numberOfPayments;
                scalapayOptionSettings.optionID = optionID;
            }
        }
        return scalapayOptionSettings;
    },
    isCurrencyApplicable: function () {
        var applicableCurrencies = currentSite.getCustomPreferenceValue('scalaPayCurrencies');
        var isCurrencyApplicable = false;

        for (var i = 0; i < applicableCurrencies.length; i += 1) {
            if (applicableCurrencies[i].toLowerCase() == session.currency.currencyCode.toLowerCase()) {
                isCurrencyApplicable = true;
                break;
            }
        }
        return isCurrencyApplicable;
    },
    getSettingsMapping: function () {
        return settingsMapping;
    },
    isShippingCountryApplicable: function (basket) {
        var isShippingCountryApplicable = true;

        if (basket) {
            var shipment = basket.getDefaultShipment();

            if (shipment) {
                var shippingAddress = shipment.getShippingAddress();

                if (shippingAddress) {
                    var shippingCountry = shippingAddress.countryCode ? shippingAddress.countryCode.value : null;

                    if (shippingCountry) {
                        var applicableCountries = currentSite.getCustomPreferenceValue('scalaPayCountries');

                        if (applicableCountries) {
                            var isCountryInList = false;

                            for (var i = 0; i < applicableCountries.length; i += 1) {
                                var applicableCountryCode = applicableCountries[i].toLowerCase();

                                if (shippingCountry.toLowerCase() == applicableCountryCode) {
                                    isCountryInList = true;
                                    break;
                                }
                            }
                            isShippingCountryApplicable = isCountryInList;
                        }
                    }
                }
            }
        }
        return isShippingCountryApplicable;
    },
    checkPaymentToken: function (token, basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        var paymentInstrument = this.getPaymentInstr(basket);

        if (paymentInstrument) {
            var paymentTransaction = paymentInstrument.getPaymentTransaction();

            if (paymentTransaction && paymentTransaction.transactionID === token) {
                return true;
            }
        }
        return false;
    },
    isScalaPayMethod: function (basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        return this.getPaymentInstr(basket);
    },
    isPaymentApplicable: function (paymentOptionID, basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        var basketTotalVal = scalapayUtils.getNonGiftCertificateAmount(basket).value;
        var isApplicable = false;

        var isCountryApplicable = this.isShippingCountryApplicable(basket);
        if (isCountryApplicable) {
            isApplicable = this.getScalaPayOptionSettings(basketTotalVal, null, paymentOptionID).isEligible;
        }

        return isApplicable;
    },
    getPaymentReqData: function (basket, paymentOptionID) {
        var handlePaymentData;

        if (basket) {
            var URLUtils = require('dw/web/URLUtils');

            handlePaymentData = {
                totalAmount: {
                    amount: scalapayUtils.getNonGiftCertificateAmount(basket).value.toString(),
                    currency: session.currency.currencyCode
                },
                consumer: getCustomerData(basket),
                billing: getAddressData(basket.billingAddress),
                shipping: getAddressData(basket.defaultShipment.shippingAddress),
                items: getItemsData(basket),
                merchant: {
                    redirectConfirmUrl: URLUtils.https('Scalapay-HandlePayment').toString(),
                    redirectCancelUrl: URLUtils.https('Checkout-Begin', 'stage', 'payment').toString()
                },
                taxAmount: {
                    amount: basket.getTotalTax().value.toString(),
                    currency: session.currency.currencyCode
                },
                shippingAmount: {
                    amount: basket.getShippingTotalPrice().value.toString(),
                    currency: session.currency.currencyCode
                },
                type: 'online',
                product: getPaymentProductType(paymentOptionID),
                frequency: getPaymentFrequencySettings(paymentOptionID),
                orderExpiryMilliseconds: 3600000
            };
            var basketDiscounts = getPriceAdjustmentData(basket);
            if (basketDiscounts && basketDiscounts.length) {
                handlePaymentData.discounts = basketDiscounts;
            }
        }

        return handlePaymentData;
    },
    isPaymentDataSync: function (basket) {
        var BasketMgr = require('dw/order/BasketMgr');

        basket = (basket || BasketMgr.getCurrentOrNewBasket());

        var paymentInstrument = this.getPaymentInstr(basket),
            isPaymentSync = false;

        if (paymentInstrument) {
            var paymentTransaction = paymentInstrument.getPaymentTransaction();

            if (paymentTransaction) {
                var authorizedAmount = paymentTransaction.custom ? paymentTransaction.custom.authorizedAmount : 0,
                    orderAmount = scalapayUtils.getNonGiftCertificateAmount(basket).value;

                isPaymentSync = (+authorizedAmount === +orderAmount);
            }
        }
        return isPaymentSync;
    },
    getPaymentInstr: function (basket) {
        var PaymentMgr = require('dw/order/PaymentMgr'),
            paymentInstruments = basket.getPaymentInstruments(),
            scalaPayPI;

        if (paymentInstruments && paymentInstruments.size()) {
            for (var i = 0; i < paymentInstruments.size(); i += 1) {
                var PI = paymentInstruments[i];

                if (PaymentMgr.getPaymentMethod(PI.getPaymentMethod()).paymentProcessor.ID.toLowerCase() == 'scalapay') {
                    scalaPayPI = PI;
                    break;
                }
            }
        }
        return scalaPayPI;
    },
    updatePaymentToken: function (token, basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        var scalaPayPI = this.getPaymentInstr(basket);

        if (scalaPayPI && token) {
            var Transaction = require('dw/system/Transaction');

            Transaction.wrap(function () {
                scalaPayPI.paymentTransaction.setTransactionID(token);
            });
        }
    },
    setAuthAmount: function (amount, basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        var scalaPayPI = this.getPaymentInstr(basket);
        if (scalaPayPI) {
            var Transaction = require('dw/system/Transaction');
            Transaction.wrap(function () {
                scalaPayPI.paymentTransaction.custom.authorizedAmount = amount;
            });
        }
    },
    getPaymentToken: function (basket) {
        var BasketMgr = require('dw/order/BasketMgr');
        basket = (basket || BasketMgr.getCurrentOrNewBasket());
        var scalaPayPI = this.getPaymentInstr(basket);
        var token = '';

        if (scalaPayPI) {
            token = scalaPayPI.paymentTransaction.getTransactionID();
        }
        return token;
    },
    getAddressSimpleObj: function (address) {
        var addressObj = {};

        if (address) {
            addressObj.city = address.city;
            addressObj.postalCode = address.postalCode;
            addressObj.countryCode = address.countryCode.value;
            addressObj.address1 = address.address1;
            addressObj.address2 = (address.address2 || '');
            addressObj.stateCode = (address.stateCode || '');
            addressObj.firstName = address.firstName;
            addressObj.lastName = address.lastName;
        }

        return addressObj;
    },
    getAuthAddresses: function (basket) {
        var billingAddress = basket.getBillingAddress(),
            defaultShipment = basket.getDefaultShipment(),
            shippingAddress = (defaultShipment ? defaultShipment.shippingAddress : null),
            that = this,
            addressData;

        if (billingAddress && shippingAddress) {
            addressData = {
                billing: that.getAddressSimpleObj(billingAddress),
                shipping: that.getAddressSimpleObj(shippingAddress)
            };
        }
        return addressData;
    },
    saveAuthAddressData: function (basket) {
        var paymentInstrument = this.getPaymentInstr(basket);

        if (paymentInstrument) {
            var paymentTransaction = paymentInstrument.getPaymentTransaction();

            if (paymentTransaction) {
                var basketAddresses = this.getAuthAddresses(basket);

                if (basketAddresses) {
                    var Transaction = require('dw/system/Transaction');
                    Transaction.wrap(function () {
                        paymentTransaction.custom.authAddressData = JSON.stringify(basketAddresses);
                    });
                }
            }
        }
    },
    compareAddresses: function (addressA, addressB) {
        var isAddressEquals = true;

        if (Object.keys(addressA).length != Object.keys(addressB).length) {
            return false;
        }
        for (var key in addressA) {
            if ((addressA[key] + '') !== (addressB[key] + '')) {
                isAddressEquals = false;
                break;
            }
        }

        return isAddressEquals;
    },
    isAddressDataSync: function (order) {
        var paymentInstrument = this.getPaymentInstr(order),
            isAddressSyncronized = false,
            Logger = require('dw/system/Logger').getLogger('scalapay');

        if (paymentInstrument) {
            var paymentTransaction = paymentInstrument.getPaymentTransaction();

            if (paymentTransaction) {
                var basketAddresses = this.getAuthAddresses(order);

                if (basketAddresses) {
                    var authAddressData = paymentTransaction.custom.authAddressData;

                    if (authAddressData && authAddressData.length) {
                        try {
                            var authAddressDataParsed = JSON.parse(authAddressData),
                                isBillingSync = this.compareAddresses(authAddressDataParsed.billing, basketAddresses.billing),
                                isShippingSync = this.compareAddresses(authAddressDataParsed.shipping, basketAddresses.shipping);

                            isAddressSyncronized = (isBillingSync && isShippingSync);
                        } catch (error) {
                            Logger.error('ScalaPay authorization address parse error');
                        }
                    }
                }
            }
        }
        return isAddressSyncronized;
    },
    isProductCategoryEligible: function (product) {
        var isEligible = true;

        if (product) {
            var masterProduct = product.isVariant() ? product.getVariationModel().master : product;
            var productCategory = masterProduct.getPrimaryCategory();

            if (productCategory && productCategory.custom && 'scalaPayDisabled' in productCategory.custom) {
                isEligible = !(productCategory.custom.scalaPayDisabled);
            }
        }

        return isEligible;
    },
    submitOrder: function (order) {
        if (order) {
            var isDelayCaptureEnabled = currentSite.getCustomPreferenceValue('scalapayDelayCaptureEnabled');

            if (!isDelayCaptureEnabled) {
                var Transaction = require('dw/system/Transaction');
                var PAID_STATUS = require('dw/order/Order').PAYMENT_STATUS_PAID;

                Transaction.wrap(function () {
                    order.setPaymentStatus(PAID_STATUS);
                });
            }

            if (session.privacy.scalapayAuthURL) {
                delete session.privacy.scalapayAuthURL;
            }
        }
    }
};

module.exports = ScalaPayHelper;
