'use strict';

var Site = require('dw/system/Site');
var currentSite = Site.getCurrent();

/**
 * @description remove required payment instrument from the Basket
 * @param {dw.order.Basket} basket SFCC Basket object
 * @param {Array} methods payment method IDs
 * @returns {void}
 */
function removeExistingPaymentInstruments(basket, methods) {
    methods.forEach(function (methodID) {
        var iter = basket.getPaymentInstruments(methodID).iterator();

        // Remove payment instruments.
        while (iter.hasNext()) {
            basket.removePaymentInstrument(iter.next());
        }
    });
}

/**
 * @description // Calculate ScalaPay payment authorize service.
 * If "scalapayDelayCaptureEnabled" set to true - payment capturing will be delayed
 * @param {string} paymentToken Order payment authprize token
 * @returns {dw.svc.Service} authorization service
 */
function getScalaPayPaymentAuthorizeService(paymentToken) {
    var scalapayServices = require('*/cartridge/scripts/service/scalapayment');
    var isDelayCaptureEnabled = currentSite.getCustomPreferenceValue('scalapayDelayCaptureEnabled');
    var paymentAuthorizeService;

    if (isDelayCaptureEnabled) {
        paymentAuthorizeService = scalapayServices.ScalaPayAuthorizeDelayService;
        var initialServiceURL = paymentAuthorizeService.getURL();
        var personalizedServiceURL = initialServiceURL.replace('{token}', paymentToken);
        paymentAuthorizeService.setURL(personalizedServiceURL);
    } else {
        paymentAuthorizeService = scalapayServices.ScalaPayAuthorizeService;
    }
    return paymentAuthorizeService;
}

/**
 * @description Calculate payment authorization request data based on "scalapayDelayCaptureEnabled" preference.
 * @param {string} orderNumber SFCC order number
 * @param {string} paymentToken Payment authorization token
 * @returns {Object} request data object
 */
function getAuthorizationReqData(orderNumber, paymentToken) {
    var isDelayCaptureEnabled = currentSite.getCustomPreferenceValue('scalapayDelayCaptureEnabled');
    var reqData = {
        merchantReference: orderNumber
    };

    if (!isDelayCaptureEnabled) {
        reqData.token = paymentToken;
    }

    return reqData;
}

/**
 * @description get site payment authorization token
 * @returns {string} payment auth token
 */
function getPaymentAuthToken() {
    var mode = currentSite.getCustomPreferenceValue('scalaPayPaymentInterface');
    var tokenPrefKey;
    var authToken;

    if (mode == 'prod') {
        tokenPrefKey = 'scalaPayPaymentTokenProd';
    } else {
        tokenPrefKey = 'scalaPayPaymentTokenTest';
    }
    authToken = currentSite.getCustomPreferenceValue(tokenPrefKey);

    return authToken;
}

/**
 * @description Set scalapay payment payment transaction
 * @param {dw.order.Basket} basket SFCC Basket object
 * @param {Object} paymenInformation credit card payment information
 * @param {Object} billingData billing form payment data
 * @returns {Object} handle payment status object
 */
function handle(basket) {
    var scalapayHelper = require('*/cartridge/scripts/helpers/scalapayHelper'),
        Transaction = require('dw/system/Transaction'),
        Resource = require('dw/web/Resource'),
        Logger = require('dw/system/Logger').getLogger('scalapay'),
        paymentOptionID = session.forms.scalapaymentoptions.paymentoptions.htmlValue,
        isScalapayApplicable = scalapayHelper.isPaymentApplicable(paymentOptionID, basket),
        handlePaymentStatus = {
            error: true
        };

    // no actions if payment authorized
    if (isScalapayApplicable && session.privacy.scalapayAuthURL && scalapayHelper.getPaymentToken(basket) && scalapayHelper.isPaymentDataSync(basket)) {
        handlePaymentStatus.error = false;
        return handlePaymentStatus;
    }

    if (isScalapayApplicable) {
        var handlePaymentData = scalapayHelper.getPaymentReqData(basket, paymentOptionID),
            scalapayService = require('*/cartridge/scripts/service/scalapayment'),
            scalapayUtils = require('*/cartridge/scripts/helpers/scalapayUtils'),
            authToken = getPaymentAuthToken(),
            authAmountMoney = scalapayUtils.getNonGiftCertificateAmount(basket);

        Transaction.wrap(function () {
            removeExistingPaymentInstruments(basket, ['CREDIT_CARD', 'SCALAPAY']);
            basket.createPaymentInstrument('SCALAPAY', authAmountMoney);
        });
        if (authToken) {
            session.privacy.scalapayAuthURL = null;
            var handlePaymentResult = scalapayService.ScalaPayHandleService.call(handlePaymentData, authToken);

            if (handlePaymentResult && handlePaymentResult.ok) {
                try {
                    var responseData = JSON.parse(handlePaymentResult.object);

                    if (responseData.checkoutUrl) {
                        handlePaymentStatus.error = false;
                        session.privacy.scalapayAuthURL = responseData.checkoutUrl;
                        scalapayHelper.setAuthAmount(authAmountMoney.value, basket);
                        scalapayHelper.saveAuthAddressData(basket);
                    } else {
                        Logger.error('ScalaPay payment authorization error. Error: Payment authorization redirect URL not in response');
                    }
                } catch (error) {
                    Logger.error('ScalaPay payment authorization error. Error: {0}', error);
                }
            } else {
                handlePaymentStatus.serverErrors = Resource.msg('error.technical', 'checkout', null);
                Logger.error('ScalaPay payment authorization error. Error: {0}', handlePaymentResult.errorMessage);
            }
        } else {
            Logger.error('ScalaPay payment authorization error. Error: Authorization token missed');
        }
    } else {
        handlePaymentStatus.serverErrors = Resource.msg('scalapay.payment.disabled', 'payment', null);
        Logger.error('ScalaPay payment authorization error. Error: Payment method is not applicable');
    }

    return handlePaymentStatus;
}

/**
 * @description Authorize scalapay payment transaction
 * @param {string} orderNumber SFCC Order number
 * @param {dw.order.PaymentInstrument} paymentInstrument SFCC order payment instrument
 * @param {dw.order.PaymentProcessor} paymentProcessor SFCC order payment processor
 * @returns {Object} authorization status object
 */
function authorize(orderNumber, paymentInstrument, paymentProcessor) {
    var scalapayHelper = require('*/cartridge/scripts/helpers/scalapayHelper'),
        OrderMgr = require('dw/order/OrderMgr'),
        Transaction = require('dw/system/Transaction'),
        Logger = require('dw/system/Logger').getLogger('scalapay'),
        order = OrderMgr.getOrder(orderNumber),
        paymentOptionID = session.forms.scalapaymentoptions.paymentoptions.htmlValue,
        isScalapayApplicable = scalapayHelper.isPaymentApplicable(paymentOptionID, order),
        AuthorizePaymentStatus = {
            error: true
        };

    Transaction.wrap(function () {
        paymentInstrument.paymentTransaction.setPaymentProcessor(paymentProcessor);
    });
    if (isScalapayApplicable) {
        var authToken = getPaymentAuthToken();

        if (authToken) {
            var paymentToken = scalapayHelper.getPaymentToken(order),
                scalapayAuthorizeService = getScalaPayPaymentAuthorizeService(paymentToken),
                reqData = getAuthorizationReqData(orderNumber, paymentToken),
                isPaymentDataSyncronized = scalapayHelper.isPaymentDataSync(order),
                isAddressDataSyncronized = scalapayHelper.isAddressDataSync(order);

            if (isPaymentDataSyncronized && isAddressDataSyncronized) {
                var authResult = scalapayAuthorizeService.call(reqData, authToken);

                if (authResult && authResult.ok) {
                    AuthorizePaymentStatus.error = false;
                    scalapayHelper.submitOrder(order);
                } else {
                    Logger.error('Scalapay payment capture error. Error: Service error');
                }
            } else {
                Logger.error('Scalapay payment capture error. Error: Order payment and/or address data was modified');
            }
        } else {
            Logger.error('Scalapay payment capture error. Error: Authorization token missed');
        }
    } else {
        Logger.error('Scalapay payment capture error. Error: Scalapay is not applicable');
    }

    if (AuthorizePaymentStatus.error) {
        session.privacy.scalapayAuthURL = null;
    }
    return AuthorizePaymentStatus;
}

/**
 * Verifies the required information for billing form is provided.
 * @param {Object} req - The request object
 * @param {Object} paymentForm - the payment form
 * @param {Object} viewFormData - object contains billing form data
 * @returns {Object} an object that has error information or payment information
 */
function processForm(req, paymentForm, viewFormData) {
    var viewData = viewFormData;
    viewData.paymentMethod = {
        value: paymentForm.paymentMethod.value,
        htmlName: paymentForm.paymentMethod.value
    };
    return { fieldErrors: [], error: false, viewData: viewData };
}

module.exports = {
    Handle: handle,
    Authorize: authorize,
    processForm: processForm
};
